@@ -4,6 +4,8 @@
 
 #[cfg(feature = "dbus")]
 use dbus::ffidisp::Connection as DbusConnection;
+#[cfg(feature = "zbus")]
+use zbus::{block_on, zvariant};
 
 use crate::{error::*, notification::Notification};
 
@@ -11,18 +13,42 @@ use std::ops::{Deref, DerefMut};
 
 #[cfg(feature = "dbus")]
 mod dbus_rs;
+#[cfg(all(feature = "dbus", not(feature = "zbus")))]
+use dbus_rs::bus;
+
 #[cfg(feature = "zbus")]
 mod zbus_rs;
+#[cfg(all(feature = "zbus", not(feature = "dbus")))]
+use zbus_rs::bus;
+
+#[cfg(all(feature = "dbus", feature = "zbus"))]
+mod bus;
+
+#[cfg(all(feature = "server", feature = "dbus", unix, not(target_os = "macos")))]
+pub mod server_dbus;
+
+#[cfg(all(feature = "server", feature = "zbus", unix, not(target_os = "macos")))]
+pub mod server_zbus;
+
+#[cfg(all(feature = "server", unix, not(target_os = "macos")))]
+pub mod server;
 
 #[cfg(not(feature = "debug_namespace"))]
-pub static NOTIFICATION_NAMESPACE: &str = "org.freedesktop.Notifications";
-#[cfg(not(feature = "debug_namespace"))]
+#[doc(hidden)]
+pub static NOTIFICATION_DEFAULT_BUS: &str = "org.freedesktop.Notifications";
+
+#[cfg(feature = "debug_namespace")]
+#[doc(hidden)]
+// #[deprecated]
+pub static NOTIFICATION_DEFAULT_BUS: &str = "de.hoodie.Notifications";
+
+#[doc(hidden)]
+pub static NOTIFICATION_INTERFACE: &str = "org.freedesktop.Notifications";
+
+#[doc(hidden)]
 pub static NOTIFICATION_OBJECTPATH: &str = "/org/freedesktop/Notifications";
 
-#[cfg(feature = "debug_namespace")]
-pub static NOTIFICATION_NAMESPACE: &str = "de.hoodie.Notifications";
-#[cfg(feature = "debug_namespace")]
-pub static NOTIFICATION_OBJECTPATH: &str = "/de/hoodie/Notifications";
+pub(crate) use bus::NotificationBus;
 
 #[derive(Debug)]
 enum NotificationHandleInner {
@@ -44,14 +70,22 @@ pub struct NotificationHandle {
 #[allow(dead_code)]
 impl NotificationHandle {
     #[cfg(feature = "dbus")]
-    pub(crate) fn for_dbus(id: u32, connection: DbusConnection, notification: Notification) -> NotificationHandle {
+    pub(crate) fn for_dbus(
+        id: u32,
+        connection: DbusConnection,
+        notification: Notification,
+    ) -> NotificationHandle {
         NotificationHandle {
             inner: dbus_rs::DbusNotificationHandle::new(id, connection, notification).into(),
         }
     }
 
     #[cfg(feature = "zbus")]
-    pub(crate) fn for_zbus(id: u32, connection: zbus::blocking::Connection, notification: Notification) -> NotificationHandle {
+    pub(crate) fn for_zbus(
+        id: u32,
+        connection: zbus::Connection,
+        notification: Notification,
+    ) -> NotificationHandle {
         NotificationHandle {
             inner: zbus_rs::ZbusNotificationHandle::new(id, connection, notification).into(),
         }
@@ -65,16 +99,22 @@ impl NotificationHandle {
     {
         match self.inner {
             #[cfg(feature = "dbus")]
-            NotificationHandleInner::Dbus(inner) => inner.wait_for_action(|action: &ActionResponse| match action {
-                ActionResponse::Custom(action) => invocation_closure(action),
-                ActionResponse::Closed(_reason) => invocation_closure("__closed"), // FIXME: remove backward compatibility with 5.0
-            }),
+            NotificationHandleInner::Dbus(inner) => {
+                inner.wait_for_action(|action: &ActionResponse| match action {
+                    ActionResponse::Custom(action) => invocation_closure(action),
+                    ActionResponse::Closed(_reason) => invocation_closure("__closed"), // FIXME: remove backward compatibility with 5.0
+                });
+            }
 
             #[cfg(feature = "zbus")]
-            NotificationHandleInner::Zbus(inner) => inner.wait_for_action(|action: &ActionResponse| match action {
-                ActionResponse::Custom(action) => invocation_closure(action),
-                ActionResponse::Closed(_reason) => invocation_closure("__closed"), // FIXME: remove backward compatibility with 5.0
-            }),
+            NotificationHandleInner::Zbus(inner) => {
+                block_on(
+                    inner.wait_for_action(|action: &ActionResponse| match action {
+                        ActionResponse::Custom(action) => invocation_closure(action),
+                        ActionResponse::Closed(_reason) => invocation_closure("__closed"), // FIXME: remove backward compatibility with 5.0
+                    }),
+                );
+            }
         };
     }
 
@@ -100,7 +140,7 @@ impl NotificationHandle {
             #[cfg(feature = "dbus")]
             NotificationHandleInner::Dbus(inner) => inner.close(),
             #[cfg(feature = "zbus")]
-            NotificationHandleInner::Zbus(inner) => inner.close(),
+            NotificationHandleInner::Zbus(inner) => block_on(inner.close()),
         }
     }
 
@@ -117,7 +157,7 @@ impl NotificationHandle {
     ///                    .unwrap()
     ///                    .on_close(|| println!("closed"));
     /// ```
-    /// 
+    ///
     /// ## Example 2: *I **do** care about why it closed* (added in v4.5.0)
     ///
     /// ```no_run
@@ -132,17 +172,21 @@ impl NotificationHandle {
     pub fn on_close<A>(self, handler: impl CloseHandler<A>) {
         match self.inner {
             #[cfg(feature = "dbus")]
-            NotificationHandleInner::Dbus(inner) => inner.wait_for_action(|action: &ActionResponse| {
-                if let ActionResponse::Closed(reason) = action {
-                    handler.call(*reason);
-                }
-            }),
+            NotificationHandleInner::Dbus(inner) => {
+                inner.wait_for_action(|action: &ActionResponse| {
+                    if let ActionResponse::Closed(reason) = action {
+                        handler.call(*reason);
+                    }
+                });
+            }
             #[cfg(feature = "zbus")]
-            NotificationHandleInner::Zbus(inner) => inner.wait_for_action(|action: &ActionResponse| {
-                if let ActionResponse::Closed(reason) = action {
-                    handler.call(*reason);
-                }
-            }),
+            NotificationHandleInner::Zbus(inner) => {
+                block_on(inner.wait_for_action(|action: &ActionResponse| {
+                    if let ActionResponse::Closed(reason) = action {
+                        handler.call(*reason);
+                    }
+                }));
+            }
         };
     }
 
@@ -229,21 +273,33 @@ impl From<zbus_rs::ZbusNotificationHandle> for NotificationHandleInner {
 #[cfg(feature = "dbus")]
 impl From<dbus_rs::DbusNotificationHandle> for NotificationHandle {
     fn from(handle: dbus_rs::DbusNotificationHandle) -> NotificationHandle {
-        NotificationHandle { inner: handle.into() }
+        NotificationHandle {
+            inner: handle.into(),
+        }
     }
 }
 
 #[cfg(feature = "zbus")]
 impl From<zbus_rs::ZbusNotificationHandle> for NotificationHandle {
     fn from(handle: zbus_rs::ZbusNotificationHandle) -> NotificationHandle {
-        NotificationHandle { inner: handle.into() }
+        NotificationHandle {
+            inner: handle.into(),
+        }
     }
 }
 
 // here be public functions
 
-#[cfg(all(not(any(feature = "dbus", feature = "zbus")), unix, not(target_os = "macos")))]
-compile_error!("you have to build with eiter zbus or dbus turned on");
+// TODO: breaking change, wait for 5.0
+// #[cfg(all(feature = "dbus", feature = "zbus"))]
+//compile_error!("the z and d features are mutually exclusive");
+
+#[cfg(all(
+    not(any(feature = "dbus", feature = "zbus")),
+    unix,
+    not(target_os = "macos")
+))]
+compile_error!("you have to build with either zbus or dbus turned on");
 
 /// Which Dbus implementation are we using?
 #[derive(Copy, Clone, Debug)]
@@ -259,7 +315,26 @@ const DBUS_SWITCH_VAR: &str = "DBUSRS";
 
 #[cfg(all(feature = "zbus", not(feature = "dbus")))]
 pub(crate) fn show_notification(notification: &Notification) -> Result<NotificationHandle> {
-    zbus_rs::connect_and_send_notification(notification).map(Into::into)
+    block_on(zbus_rs::connect_and_send_notification(notification)).map(Into::into)
+}
+
+#[cfg(all(feature = "async", feature = "zbus"))]
+pub(crate) async fn show_notification_async(
+    notification: &Notification,
+) -> Result<NotificationHandle> {
+    zbus_rs::connect_and_send_notification(notification)
+        .await
+        .map(Into::into)
+}
+
+#[cfg(all(feature = "async", feature = "zbus"))]
+pub(crate) async fn show_notification_async_at_bus(
+    notification: &Notification,
+    bus: NotificationBus,
+) -> Result<NotificationHandle> {
+    zbus_rs::connect_and_send_notification_at_bus(notification, bus)
+        .await
+        .map(Into::into)
 }
 
 #[cfg(all(feature = "dbus", not(feature = "zbus")))]
@@ -272,11 +347,11 @@ pub(crate) fn show_notification(notification: &Notification) -> Result<Notificat
     if std::env::var(DBUS_SWITCH_VAR).is_ok() {
         dbus_rs::connect_and_send_notification(notification).map(Into::into)
     } else {
-        zbus_rs::connect_and_send_notification(notification).map(Into::into)
+        block_on(zbus_rs::connect_and_send_notification(notification)).map(Into::into)
     }
 }
 
-/// Get the currently dsed [`DbusStack`]
+/// Get the currently used [`DbusStack`]
 ///
 /// (zbus only)
 #[cfg(all(feature = "zbus", not(feature = "dbus")))]
@@ -284,7 +359,7 @@ pub fn dbus_stack() -> Option<DbusStack> {
     Some(DbusStack::Zbus)
 }
 
-/// Get the currently dsed [`DbusStack`]
+/// Get the currently used [`DbusStack`]
 ///
 /// (dbus-rs only)
 #[cfg(all(feature = "dbus", not(feature = "zbus")))]
@@ -292,7 +367,7 @@ pub fn dbus_stack() -> Option<DbusStack> {
     Some(DbusStack::Dbus)
 }
 
-/// Get the currently dsed [`DbusStack`]
+/// Get the currently used [`DbusStack`]
 ///
 /// both dbus-rs and zbus, switch via `$ZBUS_NOTIFICATION`
 #[cfg(all(feature = "dbus", feature = "zbus"))]
@@ -304,7 +379,7 @@ pub fn dbus_stack() -> Option<DbusStack> {
     })
 }
 
-/// Get the currently dsed [`DbusStack`]
+/// Get the currently used [`DbusStack`]
 ///
 /// neither zbus nor dbus-rs are configured
 #[cfg(all(not(feature = "dbus"), not(feature = "zbus")))]
@@ -317,7 +392,7 @@ pub fn dbus_stack() -> Option<DbusStack> {
 /// (zbus only)
 #[cfg(all(feature = "zbus", not(feature = "dbus")))]
 pub fn get_capabilities() -> Result<Vec<String>> {
-    zbus_rs::get_capabilities()
+    block_on(zbus_rs::get_capabilities())
 }
 
 /// Get list of all capabilities of the running notification server.
@@ -336,7 +411,7 @@ pub fn get_capabilities() -> Result<Vec<String>> {
     if std::env::var(DBUS_SWITCH_VAR).is_ok() {
         dbus_rs::get_capabilities()
     } else {
-        zbus_rs::get_capabilities()
+        block_on(zbus_rs::get_capabilities())
     }
 }
 
@@ -348,7 +423,7 @@ pub fn get_capabilities() -> Result<Vec<String>> {
 /// (zbus only)
 #[cfg(all(feature = "zbus", not(feature = "dbus")))]
 pub fn get_server_information() -> Result<ServerInformation> {
-    zbus_rs::get_server_information()
+    block_on(zbus_rs::get_server_information())
 }
 
 /// Returns a struct containing `ServerInformation`.
@@ -373,14 +448,14 @@ pub fn get_server_information() -> Result<ServerInformation> {
     if std::env::var(DBUS_SWITCH_VAR).is_ok() {
         dbus_rs::get_server_information()
     } else {
-        zbus_rs::get_server_information()
+        block_on(zbus_rs::get_server_information())
     }
 }
 
 /// Return value of `get_server_information()`.
 #[derive(Debug)]
 #[cfg_attr(feature = "serde", derive(serde::Deserialize))]
-#[cfg_attr(feature = "zbus", derive(zvariant_derive::Type))]
+#[cfg_attr(feature = "zbus", derive(zvariant::Type))]
 pub struct ServerInformation {
     /// The product name of the server.
     pub name: String,
@@ -412,7 +487,7 @@ pub fn handle_action<F>(id: u32, func: F)
 where
     F: FnOnce(&ActionResponse),
 {
-    zbus_rs::handle_action(id, func)
+    block_on(zbus_rs::handle_action(id, func));
 }
 
 /// Listens for the `ActionInvoked(UInt32, String)` Signal.
@@ -425,7 +500,7 @@ pub fn handle_action<F>(id: u32, func: F)
 where
     F: FnOnce(&ActionResponse),
 {
-    dbus_rs::handle_action(id, func)
+    dbus_rs::handle_action(id, func);
 }
 
 /// Listens for the `ActionInvoked(UInt32, String)` Signal.
@@ -439,13 +514,13 @@ where
     F: FnOnce(&ActionResponse),
 {
     if std::env::var(DBUS_SWITCH_VAR).is_ok() {
-        dbus_rs::handle_action(id, func)
+        dbus_rs::handle_action(id, func);
     } else {
-        zbus_rs::handle_action(id, func)
+        block_on(zbus_rs::handle_action(id, func));
     }
 }
 
-/// Reased passed to `NotificationClosed` Signal
+/// Reason passed to `NotificationClosed` Signal
 ///
 /// ## Specification
 /// As listed under [Table 8. `NotificationClosed` Parameters](https://specifications.freedesktop.org/notification-spec/latest/ar01s09.html#idm46350804042704)
@@ -483,7 +558,7 @@ where
     F: FnOnce(&ActionResponse),
 {
     fn call(self, res: &ActionResponse) {
-        (self)(res)
+        (self)(res);
     }
 }
 
@@ -516,7 +591,7 @@ where
     F: Fn(CloseReason),
 {
     fn call(&self, reason: CloseReason) {
-        self(reason)
+        self(reason);
     }
 }
 
@@ -525,6 +600,6 @@ where
     F: Fn(),
 {
     fn call(&self, _: CloseReason) {
-        self()
+        self();
     }
 }
